@@ -30,18 +30,19 @@
 #include <gdbus/gdbus.h>
 
 #include "lib/uuid.h"
-#include "plugin.h"
-#include "adapter.h"
-#include "device.h"
-#include "profile.h"
-#include "service.h"
-#include "dbus-common.h"
-#include "error.h"
+#include "src/plugin.h"
+#include "src/adapter.h"
+#include "src/device.h"
+#include "src/profile.h"
+#include "src/service.h"
+#include "src/dbus-common.h"
+#include "src/shared/util.h"
+#include "src/error.h"
 #include "attrib/gattrib.h"
 #include "attrib/att.h"
 #include "attrib/gatt.h"
-#include "attio.h"
-#include "log.h"
+#include "src/attio.h"
+#include "src/log.h"
 
 /* min length for ATT indication or notification: opcode (1b) + handle (2b) */
 #define ATT_HDR_LEN 3
@@ -380,7 +381,7 @@ static void read_feature_cb(guint8 status, const guint8 *pdu, guint16 len,
 		return;
 	}
 
-	csc->feature = att_get_u16(value);
+	csc->feature = get_le16(value);
 
 	if ((csc->feature & MULTI_SENSOR_LOC_SUPPORT)
 						&& (csc->locations == NULL))
@@ -418,13 +419,12 @@ static void read_location_cb(guint8 status, const guint8 *pdu,
 					CYCLINGSPEED_INTERFACE, "Location");
 }
 
-static void discover_desc_cb(guint8 status, const guint8 *pdu,
-					guint16 len, gpointer user_data)
+static void discover_desc_cb(guint8 status, GSList *descs, gpointer user_data)
 {
 	struct characteristic *ch = user_data;
-	struct att_data_list *list = NULL;
-	uint8_t format;
-	int i;
+	struct gatt_desc *desc;
+	uint8_t attr_val[2];
+	char *msg = NULL;
 
 	if (status != 0) {
 		error("Discover %s descriptors failed: %s", ch->uuid,
@@ -432,55 +432,31 @@ static void discover_desc_cb(guint8 status, const guint8 *pdu,
 		goto done;
 	}
 
-	list = dec_find_info_resp(pdu, len, &format);
-	if (list == NULL)
-		goto done;
+	/* There will be only one descriptor on list and it will be CCC */
+	desc = descs->data;
 
-	if (format != ATT_FIND_INFO_RESP_FMT_16BIT)
-		goto done;
+	if (g_strcmp0(ch->uuid, CSC_MEASUREMENT_UUID) == 0) {
+		ch->csc->measurement_ccc_handle = desc->handle;
 
-	for (i = 0; i < list->num; i++) {
-		uint8_t *value;
-		uint16_t handle, uuid;
-		uint8_t attr_val[2];
-		char *msg;
-
-		value = list->data[i];
-		handle = att_get_u16(value);
-		uuid = att_get_u16(value + 2);
-
-		if (uuid != GATT_CLIENT_CHARAC_CFG_UUID)
-			continue;
-
-		if (g_strcmp0(ch->uuid, CSC_MEASUREMENT_UUID) == 0) {
-			ch->csc->measurement_ccc_handle = handle;
-
-			if (g_slist_length(ch->csc->cadapter->watchers) == 0) {
-				att_put_u16(0x0000, attr_val);
-				msg = g_strdup("Disable measurement");
-			} else {
-				att_put_u16(GATT_CLIENT_CHARAC_CFG_NOTIF_BIT,
-								attr_val);
-				msg = g_strdup("Enable measurement");
-			}
-
-		} else if (g_strcmp0(ch->uuid, SC_CONTROL_POINT_UUID) == 0) {
-			att_put_u16(GATT_CLIENT_CHARAC_CFG_IND_BIT, attr_val);
-			msg = g_strdup("Enable SC Control Point indications");
+		if (g_slist_length(ch->csc->cadapter->watchers) == 0) {
+			put_le16(0x0000, attr_val);
+			msg = g_strdup("Disable measurement");
 		} else {
-			break;
+			put_le16(GATT_CLIENT_CHARAC_CFG_NOTIF_BIT,
+							attr_val);
+			msg = g_strdup("Enable measurement");
 		}
-
-		gatt_write_char(ch->csc->attrib, handle, attr_val,
-					sizeof(attr_val), char_write_cb, msg);
-
-		/* We only want CCC, can break here */
-		break;
+	} else if (g_strcmp0(ch->uuid, SC_CONTROL_POINT_UUID) == 0) {
+		put_le16(GATT_CLIENT_CHARAC_CFG_IND_BIT, attr_val);
+		msg = g_strdup("Enable SC Control Point indications");
+	} else {
+		goto done;
 	}
 
+	gatt_write_char(ch->csc->attrib, desc->handle, attr_val,
+					sizeof(attr_val), char_write_cb, msg);
+
 done:
-	if (list)
-		att_data_list_free(list);
 	g_free(ch);
 }
 
@@ -489,6 +465,7 @@ static void discover_desc(struct csc *csc, struct gatt_char *c,
 {
 	struct characteristic *ch;
 	uint16_t start, end;
+	bt_uuid_t uuid;
 
 	start = c->value_handle + 1;
 
@@ -506,7 +483,10 @@ static void discover_desc(struct csc *csc, struct gatt_char *c,
 	ch->csc = csc;
 	memcpy(ch->uuid, c->uuid, sizeof(c->uuid));
 
-	gatt_discover_char_desc(csc->attrib, start, end, discover_desc_cb, ch);
+	bt_uuid16_create(&uuid, GATT_CLIENT_CHARAC_CFG_UUID);
+
+	gatt_discover_desc(csc->attrib, start, end, &uuid, discover_desc_cb,
+									ch);
 }
 
 static void update_watcher(gpointer data, gpointer user_data)
@@ -573,8 +553,8 @@ static void process_measurement(struct csc *csc, const uint8_t *pdu,
 		}
 
 		m.has_wheel_rev = true;
-		m.wheel_rev = att_get_u32(pdu);
-		m.last_wheel_time = att_get_u16(pdu + 4);
+		m.wheel_rev = get_le32(pdu);
+		m.last_wheel_time = get_le16(pdu + 4);
 		pdu += 6;
 		len -= 6;
 	}
@@ -586,8 +566,8 @@ static void process_measurement(struct csc *csc, const uint8_t *pdu,
 		}
 
 		m.has_crank_rev = true;
-		m.crank_rev = att_get_u16(pdu);
-		m.last_crank_time = att_get_u16(pdu + 2);
+		m.crank_rev = get_le16(pdu);
+		m.last_crank_time = get_le16(pdu + 2);
 		pdu += 4;
 		len -= 4;
 	}
@@ -816,7 +796,7 @@ static void enable_measurement(gpointer data, gpointer user_data)
 	if (csc->attrib == NULL || !handle)
 		return;
 
-	att_put_u16(GATT_CLIENT_CHARAC_CFG_NOTIF_BIT, value);
+	put_le16(GATT_CLIENT_CHARAC_CFG_NOTIF_BIT, value);
 	msg = g_strdup("Enable measurement");
 
 	gatt_write_char(csc->attrib, handle, value, sizeof(value),
@@ -833,7 +813,7 @@ static void disable_measurement(gpointer data, gpointer user_data)
 	if (csc->attrib == NULL || !handle)
 		return;
 
-	att_put_u16(0x0000, value);
+	put_le16(0x0000, value);
 	msg = g_strdup("Disable measurement");
 
 	gatt_write_char(csc->attrib, handle, value, sizeof(value),
@@ -1166,7 +1146,7 @@ static DBusMessage *set_cumulative_wheel_rev(DBusConnection *conn,
 	csc->pending_req = req;
 
 	att_val[0] = SET_CUMULATIVE_VALUE;
-	att_put_u32(value, att_val + 1);
+	put_le32(value, att_val + 1);
 
 	gatt_write_char(csc->attrib, csc->controlpoint_val_handle, att_val,
 		sizeof(att_val), controlpoint_write_cb, req);
